<?php
namespace app\src\controllers;

use app\src\core\BaseController;
use app\src\core\Request;
use app\src\core\middlewares\AuthMiddleware;
use app\src\core\Response;
use app\src\core\Application;
use app\src\models\User;
use app\src\models\LoginForm;
use app\src\core\helper\Mail\Email;
use app\src\core\exception\NotFoundException;
use Exception;

class AuthController extends BaseController{

    public function __construct()
    {
        $this->registerMiddleware(new AuthMiddleware(['profile']));
    }
     public function login(Request $request,Response $response){
      $loginForm = new LoginForm();
      $data = $request->getBody();
      if ($request->isPost()) {
          if ($loginForm->validation($data) && $loginForm->login($data)) {
            return $response->json(200,"Logged in successfully",['redirect'=>'/home']);
          }else{
            return $response->json(400,"error",$loginForm->errors);
          } 
      }
        return $this->render('login',[],'auth');
     }

    public function register(Request $request ,Response $response){
        try{
            $user = new User();
            $user_type = $user->getUserType();
            $data = $request->getBody();
            if($request->isPost()){
                if($user->validation($data) && $user->saveUser($data)){
                    $mail = new Email();
                    if($mail->sendMail($data['email'])){
                        return $response->json(200,"You will receive a mail with a link to activate registration. Please click on the link to activate your account");
                    }else{
                        return $response->json(200,"Something wrong with your mail id");
                    }
                }else{
                    return $response->json(400,"error",$user->errors);
                }     
            }
            return  $this->render("register",['user_types'=>$user_type],'auth');
        }catch(Exception $e){
            return $response->json(500,"Internal server error");
        }
      
    }

    public function verifyEmail(Request $request ,Response $response){
        try{
            $data = $request->getBody();
            $user = new User();
            if($request->isPost()){
                $validation_data = ["password"=>$data['password'],"confirm_password"=>$data['confirm_password']];
                if($user->validation($validation_data) && $user->activateUserWithPassword($data)){
                    return $response->json(200,"Your account has been activated.",['redirect'=>'/login']);
                }else{
                    return $response->json(400,"error",$user->errors);
                }     
            }else{
                $final_data = $user->verifyLink($data);
                if($final_data && $final_data[0]['verification_status'] != 1){
                    return  $this->render("verify",['email'=>$final_data[0]['email']],'auth');
                }else{
                    throw new NotFoundException();
                }   
            }
            
        }catch(Exception $e){
            echo Application::$app->router->renderView('_error', [
                'exception' => $e,
            ],'auth');
        }
      
    }

     public function logout(Request $request, Response $response)
    {
        Application::$app->logout();
        $response->redirect('/home');
    }
    public function profile()
    {
        return $this->render('profile');
    }
}


?>