<?php
namespace app\src\controllers;
use app\src\core\BaseController;
use app\src\core\Request;
use app\src\core\Application;
use app\src\core\helper\FileUpload\FileUpload;
use app\src\core\middlewares\AuthMiddleware;
use app\src\core\Response;
use app\src\models\HomeModel;
use app\src\models\MasterTableModel;
use Exception;

class HomeController extends BaseController{

    public function __construct()
    {
        $this->registerMiddleware(new AuthMiddleware(['index','applicationDetails','applicationDetails']));
    }
    public function index(Request $request){
         $model = new HomeModel();
         $data = $model->getUserDashbordInformation();
        return $this->render('home',$data);
    }

    public function proceed(Request $request,Response $response){
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $model = new HomeModel();
        $masterModel = new MasterTableModel();
        $data = $model->getNewApplicationDetails($id); 
        $police_station_list = $masterModel->getPoliceStations();
        $station_name = $masterModel->getPoliceStationName($data['police_station']);
        $data['police_station'] = $station_name;
        $response = [
            "data" => $data,
            "police_station_list" => $police_station_list
        ];
        return $this->render('proceed',$response);
    }

    function proceedLicense(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->proceedLicense($data);
                return $response->json(200,"Zone has been changed.",['redirect'=>'/']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function applicationDetails(Request $request,Response $response){
        $model = new HomeModel();
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $applicationDetails = $model->getApplicationDetails($id);
        return $this->render('application_details',$applicationDetails);
    }

    public function userApplicationDetails(Request $request,Response $response){
        $model = new HomeModel();
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $applicationDetails = $model->getUserApplicationDetails($id);
        return $this->render('user_theater_app_details',$applicationDetails);
    }

    public function addRemarks(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->addRemarks($data);
                return $response->json(200,"Remarks added ",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
        
    }

    public function addQuery(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->addQuery($data);
                return $response->json(200,"Query has been submitted ",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function forward(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->forwardApplication($data);
                return $response->json(200,"Appication has been forwarded successfully",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function addMemo(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->addMemo($data);
                return $response->json(200,"Memo added ",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function viewHistoryMemo(Request $request,Response $response){
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $model = new HomeModel();
        $model->viewHistoryMemo($id);
    }

    public function viewMemo(Request $request,Response $response){
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $model = new HomeModel();
        $model->viewMemo($id);
    }

    public function officer_checklist(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->addOfficerCheckList($data);
                return $response->json(200,"Checklist added successsfully ",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function addDocs(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $fileObj = new FileUpload;
                $data['docs'] = $fileObj->upload($_FILES['docs']);
                if($model->addDocs($data)){
                    return $response->json(200,"Document added successsfully",['samepage'=>'']);
                }else{
                    return $response->json(400,"Something went wrong",['samepage'=>'']);
                }
                
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function removeDocs(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                if($model->removeDocs($data)){
                    return $response->json(200,"Document removed successsfully",['samepage'=>'']);
                }else{
                    return $response->json(400,"Something went wrong",['samepage'=>'']);
                }
                
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function updateDocsVerification(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                if($model->UpdateVerification($data)){
                    return $response->json(200,"Document verified successsfully",['samepage'=>'']);
                }else{
                    return $response->json(400,"Something went wrong",['samepage'=>'']);
                }
                
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function approveMemo(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                if($model->memoApproval($data)){
                    return $response->json(200,"Memo has been approved successsfully",['samepage'=>'']);
                }else{
                    return $response->json(400,"Something went wrong",['samepage'=>'']);
                }
                
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }
    
    public function addNotes(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                if($model->addNotes($data)){
                    return $response->json(200,"Notes has been Added successsfully",['samepage'=>'']);
                }else{
                    return $response->json(400,"Something went wrong",['samepage'=>'']);
                }
                
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function viewNotes(Request $request,Response $response){
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $model = new HomeModel();
        $model->viewNotes($id);
    }

    public function generate_pdf(Request $request,Response $response){
        $data = $request->getBody();
        $model = new HomeModel();
        $model->memoPreview($data);
    }

    public function addReport(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->addReport($data);
                return $response->json(200,"Report has been submitted ",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

    public function viewReports(Request $request,Response $response){
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $model = new HomeModel();
        $model->viewReports($id);
    }

    public function notes(Request $request,Response $response){
        $data = $request->getBody();
        $id = base64_decode($data['id']);
        $model = new HomeModel();
        $notes = $model->getNotesDetails($id);
        return $this->render('notes_page',["id"=>$id,"enc_id" =>$data['id'] ,"notes"=>$notes ]);
    }

    public function addMultiNotes(Request $request,Response $response){
        $data = $request->getBody();
        $model = new HomeModel();
        $model->addMultiNotes($data);
        $id = base64_encode($data['license_id']);
        $response->redirect("applicationDetails?id=".$id);
    }

    public function reports(Request $request,Response $response){
        $data = $request->getBody();
        $model = new HomeModel();
        if ($request->isPost()) {
            $model->insertReport($data);
            $id = base64_encode($data['license_id']);
            $response->redirect("applicationDetails?id=".$id); 
        }else{
            $license_id = base64_decode($data['id']);
            $isAllReportField = $model->isAllReport();
            $reports = $model->getReportDetails($license_id);
            return $this->render('report',["id"=>$license_id,"enc_id" =>$data['id'] ,"report"=>$reports,"isAllReport"=>$isAllReportField ]);   
        }   
    }

    public function reverseRemarks(Request $request,Response $response){
        if ($request->isPost()) {
            $data = $request->getBody();
            $model = new HomeModel();
            try{
                $model->addReverseRemarks($data);
                return $response->json(200,"Remarks added ",['samepage'=>'']);
            }catch(Exception $e){
                return $response->json(400,"error",$e->getMessage());
            }
        }
    }

}


?>