<?php
namespace app\src\models;

use app\src\core\Application;
use app\src\core\Model; 


class PersonalModel extends Model{
    

    public static function tableName(): string
    {
        return 'personal_information';
    }

    public function attributes(): array
    {
        return ['name', 'email', 'password'];
    }

   
    public function rules()
    {
        return [
            'user_id' => [self::RULE_REQUIRED],
            'name' => [self::RULE_REQUIRED],
            'parent_or_spouse' => [self::RULE_REQUIRED],
            'parent_or_spouse_name' => [self::RULE_REQUIRED],
            'email' => [self::RULE_REQUIRED, self::RULE_EMAIL, [
                self::RULE_UNIQUE, 'class' => self::class
            ]],
            'gender' => [self::RULE_REQUIRED],
            'birth_country_id' => [self::RULE_REQUIRED],
            'birth_state_id' => [self::RULE_REQUIRED],
            'birth_city_id' => [self::RULE_REQUIRED],
            'date_of_birth' => [self::RULE_REQUIRED],
            'country_born' => [self::RULE_REQUIRED],
            'pan' => [self::RULE_REQUIRED],
            'aadhaar' => [self::RULE_REQUIRED],
            'passport_number' => [self::RULE_REQUIRED],
            'passport_issue_date' => [self::RULE_REQUIRED],
            'passport_expire_date' => [self::RULE_REQUIRED],
            'passport_issue_state' => [self::RULE_REQUIRED],
            'passport_issue_country' => [self::RULE_REQUIRED],
            'passport_issue_city' => [self::RULE_REQUIRED],
            'nearest_police_station' => [self::RULE_REQUIRED],
            'passport_or_other_id' => [self::RULE_REQUIRED],
            'other_id_details' => [self::RULE_REQUIRED],
        ];
    }


    public static function findOne($where)
    {
        $tableName = SELF::tableName();
        $attributes = array_keys($where);
        $sql = implode("AND", array_map(fn($attr) => "$attr = :$attr", $attributes));
        $statement = self::prepare("SELECT * FROM $tableName WHERE $sql");
        foreach ($where as $key => $item) {
            $statement->bindValue(":$key", $item);
        }
        $statement->execute();
        return $statement->fetchObject(static::class);
    }

    public function save($data)
    {

        $tableName = $this->tableName();
        if($data['passport_or_other_id'] == 'passport'){
            $attributes = ['user_id', 'name', 'parent_or_spouse','parent_or_spouse_name','email','gender','birth_country_id',
            'birth_state_id','birth_city_id','date_of_birth','pan','aadhaar','img','passport_number','passport_issue_date','passport_expire_date',
             'passport_issue_state','passport_issue_country','passport_issue_city','nearest_police_station','passport_or_other_id','other_id_details'];
        }else{
            $attributes = ['user_id', 'name', 'parent_or_spouse','parent_or_spouse_name','email','gender','birth_country_id',
        'birth_state_id','birth_city_id','date_of_birth','pan','aadhaar','img','nearest_police_station','passport_or_other_id','other_id_details','other_id_type'];
        }
        
        $params = array_map(fn($attr) => ":$attr", $attributes);
        $statement = self::prepare("INSERT INTO $tableName (" . implode(",", $attributes) . ") 
                VALUES (" . implode(",", $params) . ")");
       
        foreach ($attributes as $attribute) {
            $statement->bindValue(":$attribute", $data[$attribute]);
        }
        $statement->execute();
        return true;
    }

    public function getPersonalDetails(){
        $user_id = Application::$app->user ? Application::$app->user->id : 0;
        $statement = self::prepare("SELECT * FROM personal_information where id =$user_id ");
        $statement->execute();
        $records = $statement->fetchAll(\PDO::FETCH_ASSOC);
        return $records;
    }

}

?>