<?php
namespace app\src\models;
use app\src\core\Application;
use \PDO;

use app\src\core\Model;


class TheaterModel extends Model{
    public $is_edit = false;
    public $licence_details_id;
    public $theaterId;

    public static function tableName(): string
    {
        return 'theatre_information';
    }

    public function getAttributesFromRequest($data): array
    {
        $attributes =[];
         
        foreach ($data as $key => $value) {
            if($key == 'edit_id' && $value != ''){
                $this->is_edit = true;
            }
            if($key != 'edit_id'){
                $attributes[] = $key;
            }
        }
       
        if(!$data['edit_id']){
            $this->licence_details_id = $this->createNewLicence($data);
            $attributes[] = "license_details_id";
        }
        return $attributes;
    }

    public function createNewLicence($data){
        $user_id = Application::$app->user? Application::$app->user->id : 0 ;
        $zone_id = $this->getZoneId($data['police_station']);
        $ref_no = $this->generateUniqueId();
        $tableName = "licence_details";
        $currentDate = date("Y-m-d");
        $attributes = ["license_department","status","user_id","licence_id",'zone_id','is_completed','type','reference_number','address','police_station',"created_at"];
        $license_data = ["license_department"=>1,"status"=>1,"user_id"=>$user_id,"licence_id"=>1,'zone_id'=>$zone_id,'is_completed'=>0,'type'=>2,'reference_number'=>$ref_no,'address'=>$data['address'],'police_station'=>$data['police_station'],'created_at'=>$currentDate];
       
        $params = array_map(function($attr) { return ":$attr"; }, $attributes);
       
        $statement = self::prepare("INSERT INTO $tableName (" . implode(",", $attributes) . ") 
                VALUES (" . implode(",", $params) . ")");
        foreach ($attributes as $attribute) {
            $statement->bindValue(":$attribute", $license_data[$attribute]);
        }
        $statement->execute();
        $lastInsertId = Application::$app->db->pdo->lastInsertId('licence_details');
        return $lastInsertId;
    }

    function generateUniqueId() {
        $timestamp = time();
        $randomNumber = mt_rand();
        $uniqueId = $timestamp . $randomNumber;
        return $uniqueId;
    }

    public function getZoneId($police_station){
        $statement = self::prepare("SELECT ps.name,dr.* FROM police_station ps INNER JOIN  department_range dr on dr.id = ps.range_id  WHERE ps.id=:police_station");
        $statement->bindValue(":police_station", $police_station);
        $statement->execute();
        $range_details = $statement->fetchAll(PDO::FETCH_ASSOC)[0];
        return $range_details['zone_id'];
    }

   
    public function rules()
    {
        return [
            'license_number' => [self::RULE_REQUIRED],
            'address' => [self::RULE_REQUIRED],
            'date_of_issue' => [self::RULE_REQUIRED],
            'last_renewal_date' => [self::RULE_REQUIRED],
            'police_station' => [self::RULE_REQUIRED],
            'check_name_address' => [self::RULE_REQUIRED],
            'check_last_renewal' => [self::RULE_REQUIRED],
            'check_fee' => [self::RULE_REQUIRED],
            'check_structural' => [self::RULE_REQUIRED],
            'check_film_division' => [self::RULE_REQUIRED],
            'check_electrical' => [self::RULE_REQUIRED],
            'check_property' => [self::RULE_REQUIRED],
            'check_entertainment' => [self::RULE_REQUIRED],
            'check_fire' => [self::RULE_REQUIRED],
            'check_affidavit' => [self::RULE_REQUIRED],
            'check_insurance_and_policy' => [self::RULE_REQUIRED],
            'check_lease_or_rental' => [self::RULE_REQUIRED],
        ];
    }


    public static function findOne($where)
    {
        $tableName = SELF::tableName();
        $attributes = array_keys($where);
        $sql = implode("AND", array_map(fn($attr) => "$attr = :$attr", $attributes));
        $statement = self::prepare("SELECT * FROM $tableName WHERE $sql");
        foreach ($where as $key => $item) {
            $statement->bindValue(":$key", $item);
        }
        $statement->execute();
        return $statement->fetchObject(static::class);
    }

    public function save($data)
    {
       try{
            if(isset($data['theater_id']) && !empty($data['theater_id'])){
                  $this->updateThaterInformation($data);
                  return true;
            }else{
                $tableName = $this->tableName();
                $attributes = $this->getAttributesFromRequest($data);
                if(!$this->is_edit){
                    $data['license_details_id'] = $this->licence_details_id;
                }
                $params = array_map(fn($attr) => ":$attr", $attributes);
                $statement = self::prepare("INSERT INTO $tableName (" . implode(",", $attributes) . ") 
                        VALUES (" . implode(",", $params) . ")");
                  
                foreach ($attributes as $attribute) {
                    $statement->bindValue(":$attribute", $data[$attribute]);
                }
                $statement->execute();
                $this->theaterId = Application::$app->db->pdo->lastInsertId('theatre_information');
                return true;
            }
            
       }catch(\Exception $e){
          return false;
       }
        
    }

    public function  updateThaterInformation($data){
        $stmt = SELF::prepare("UPDATE theatre_information SET license_number =:license_number,address=:address ,date_of_issue = :date_of_issue,
        police_station=:police_station WHERE id = :id ");
        $stmt->bindParam(':id', $data['theater_id'], PDO::PARAM_INT);
        $stmt->bindParam(':license_number', $data['license_number']);
        $stmt->bindParam(':address', $data['address']);
        $stmt->bindParam(':date_of_issue', $data['date_of_issue']);
        $stmt->bindParam(':police_station', $data['police_station']);
        $stmt->execute();
    }

    function savePayment($data){
        if(isset($data['theater_id']) && !empty($data['theater_id'])){
            $tableName = $this->tableName();
            $renewal_fee = $data['renewal_fee'];
            $late_fee = $data['late_fee'];
            $total_fee = $data['total_fee'];
           
            $statement = self::prepare("UPDATE $tableName SET renewal_fee=:renewal_fee, late_fee=:late_fee, total_fee=:total_fee WHERE id = :id");
            
            $statement->bindValue(":renewal_fee", $renewal_fee);
            $statement->bindValue(":late_fee", $late_fee);
            $statement->bindValue(":total_fee", $total_fee);
            $statement->bindValue(":id", $data["theater_id"]);
            $statement->execute();
            
            $is_query = 0;
            $licence_details_statement = self::prepare("UPDATE licence_details SET is_query=:is_query WHERE id = :id");
            $licence_details_statement->bindValue(":id", $data['license_id']);
            $licence_details_statement->bindValue(":is_query", $is_query, PDO::PARAM_INT); // Assuming is_query is an integer
            $licence_details_statement->execute();
            
           
            $is_resolved = 1;
            $licence_query_statement = self::prepare("UPDATE licence_query SET is_resolved=:is_resolved WHERE license_details_id = :id");
            $licence_query_statement->bindValue(":id", $data['license_id']);
            $licence_query_statement->bindValue(":is_resolved", $is_resolved, PDO::PARAM_INT); // Assuming is_resolved is an integer
            $licence_query_statement->execute();
            
            return true;
        }else{
            $tableName = $this->tableName();
            $attributes = $this->getAttributesFromRequest($data);
            $params = array_map(fn($attr) => "$attr=:$attr", $attributes);
            $statement = self::prepare("UPDATE $tableName SET " . implode(",", $params) . "  WHERE id = :id");
            foreach ($attributes as $attribute) {
                $statement->bindValue(":$attribute", $data[$attribute]);
            }
            $statement->bindValue(":id", $data["edit_id"]);
            $statement->execute();
            return true;
        }
        
    }

    public function docSave($data,$attributes){
        $tableName = "theatre_docs";
        $params = array_map(fn($attr) => ":$attr", $attributes);
        $statement = self::prepare("INSERT INTO $tableName (" . implode(",", $attributes) . ") 
                VALUES (" . implode(",", $params) . ")");
        foreach ($attributes as $attribute) {
            $statement->bindValue(":$attribute", $data[$attribute]);
        }
        $statement->execute();
        return true;
    }

    public function otherDocSave($documents,$theatre_information_id){
        foreach($documents as $doc){
            $stmt = SELF::prepare("INSERT INTO  other_user_docs (doc,theater_id) values (:doc,:theater_id) ");
            $stmt->bindParam(':doc', $doc);
            $stmt->bindParam(':theater_id', $theatre_information_id);
            $stmt->execute();
        } 
    }

    public function declarationSave($declaration,$theatre_information_id){
        $stmt = SELF::prepare("UPDATE theatre_information SET  declaration=:declaration WHERE id=:id");
        $stmt->bindParam(':id', $theatre_information_id);
        $stmt->bindParam(':declaration', $declaration);
        $stmt->execute();
    }

    public function updateDoc($data,$theater_id){
        
        $stmt = self::prepare("UPDATE theatre_docs SET  
        last_renewal_proceedings=:last_renewal_proceedings, fee_details=:fee_details, requisition_letter=:requisition_letter,
        certificate_obtained_from_pwd=:certificate_obtained_from_pwd, film_division_certificate=:film_division_certificate,
        electrical_certificate=:electrical_certificate, property_tax=:property_tax, entertainemnt_tax=:entertainemnt_tax,
        fire_service=:fire_service, affidavit=:affidavit, insurance_policy=:insurance_policy, lease_or_rental=:lease_or_rental,
        public_liability=:public_liability
        WHERE theatre_information_id=:theatre_information_id");

        $stmt->bindParam(':theatre_information_id', $theater_id);
        $stmt->bindParam(':last_renewal_proceedings', $data['last_renewal_proceedings']);
        $stmt->bindParam(':fee_details', $data['fee_details']);
        $stmt->bindParam(':requisition_letter', $data['declaration']);
        $stmt->bindParam(':certificate_obtained_from_pwd', $data['certificate_obtained_from_pwd']);
        $stmt->bindParam(':film_division_certificate', $data['film_division_certificate']);
        $stmt->bindParam(':electrical_certificate', $data['electrical_certificate']);
        $stmt->bindParam(':property_tax', $data['property_tax']);
        $stmt->bindParam(':entertainemnt_tax', $data['entertainemnt_tax']);
        $stmt->bindParam(':fire_service', $data['fire_service']);
        $stmt->bindParam(':affidavit', $data['affidavit']);
        $stmt->bindParam(':insurance_policy', $data['insurance_policy']);
        $stmt->bindParam(':lease_or_rental', $data['lease_or_rental']);
        $stmt->bindParam(':public_liability', $data['public_liability']);
        $stmt->execute();

    }



}

?>