<?php
namespace app\src\models;
use app\src\core\Application;
use \PDO;

use app\src\core\Model;


class User extends Model{

    public $first_name;
    public $last_name;
    public $email;
    public $password;
    public $phone;
    public $user_type;
    public $id;

    public $cps_gpf_num;
    public $verification_status;
    public $created_at;

    public $updated_at;
    public $verify_otp;

    public $status;

    

    public static function tableName(): string
    {
        return 'users';
    }

    public static function primaryKey(){
        return "id";
    }

    public function attributes(): array
    {
        return ['name', 'email', 'password'];
    }

   
    public function rules()
    {
        return [
            'first_name' => [self::RULE_REQUIRED],
            'user_type' => [self::RULE_REQUIRED],
            'phone' => [self::RULE_REQUIRED],
            'last_name' => [self::RULE_REQUIRED],
            'email' => [self::RULE_REQUIRED, self::RULE_EMAIL, [
                self::RULE_UNIQUE, 'class' => self::class
            ]],
            'password' => [self::RULE_REQUIRED, [self::RULE_MIN, 'min' => 8]],
            'confirm_password' => [[self::RULE_MATCH, 'match' => 'password']],
        ];
    }

    public function getUserType(){
        $tableName = "user_type";
        $statement = self::prepare("SELECT * FROM $tableName WHERE status = :status");
        $statement->bindValue(":status", 1);
        $statement->execute();
        $records = $statement->fetchAll(PDO::FETCH_ASSOC);
        return $records;
    }

    public static function findOne($where)
    {
        $tableName = SELF::tableName();
        $attributes = array_keys($where);
        $sql = implode(" AND ", array_map(function($attr) { return "$attr = :$attr"; }, $attributes));
        $statement = self::prepare("SELECT * FROM $tableName WHERE $sql");
        foreach ($where as $key => $item) {
            $statement->bindValue(":$key", $item);
        }
        $statement->execute();
        return $statement->fetchObject(static::class);
    }

    public function save()
    {
        $tableName = $this->tableName();
        $attributes = $this->attributes();
        $params = array_map(fn($attr) => ":$attr", $attributes);
        $statement = self::prepare("INSERT INTO $tableName (" . implode(",", $attributes) . ") 
                VALUES (" . implode(",", $params) . ")");
       
        foreach ($attributes as $attribute) {
            $statement->bindValue(":$attribute", $this->{$attribute});
        }
        $statement->execute();
        return true;
    }

    public function activateUserWithPassword($data){
        $tableName = $this->tableName();
        $statement = self::prepare("UPDATE $tableName SET password = :password, verification_status = :verification,status=:status WHERE  email=:email");
        $statement->bindValue(":password", password_hash($data['password'], PASSWORD_DEFAULT));
        $statement->bindValue(":verification", 1);
        $statement->bindValue(":status", 1);
        $statement->bindValue(":email", $data['email']);
        $statement->execute();
        return true;
    }

    public function verifyLink($data){
        $tableName = $this->tableName();
        $statement = self::prepare("SELECT * FROM $tableName WHERE MD5(email)=:email");
        $statement->bindValue(":email", $data['em']);
        $statement->execute();
        $records = $statement->fetchAll(PDO::FETCH_ASSOC);
        return $records;
    }

    function saveUser($data){
        $tableName = $this->tableName();
        $attributes = ['user_type', 'email', 'first_name','last_name','phone'];
        $params = array_map(fn($attr) => ":$attr", $attributes);
        $statement = self::prepare("INSERT INTO $tableName (" . implode(",", $attributes) . ") 
                VALUES (" . implode(",", $params) . ")");
       
        foreach ($attributes as $attribute) {
            $statement->bindValue(":$attribute", $data[$attribute]);
        }
        $statement->execute();
        return true;
    }

}

?>